/* stepper.c: a program to drive a stepping motor
 * copyright U. Raich 2017
 * This program is part of the workshop on embedded systems held at 
 * the University of Cape Coast in the winter semester 2017
 * It will be demonstrated at the University Open Day on Oct. 18 2017
 * It has been released under the GNU General Public License
 * see gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <stdbool.h>
#define DEBUG 1

static struct option long_options [] =
  {
    {"help",              no_argument,          NULL, '?'},     
    {"clockwise",         no_argument,          NULL, 'c'},      
    {"counterclockwise",  no_argument,          NULL, 'b'},
    {"noOfSteps",         required_argument,    NULL, 'n'},      
    {"velocity",          required_argument,    NULL, 'v'},      
    {"fullstep",          no_argument,          NULL, 's'},
    {"halfstep",          no_argument,          NULL, 'h'},
    {NULL,0,NULL,0}
  };

void printUsage()
{
  printf("The stepper program called without any parameter will turn\n");
  printf("the motor one full turn clockwise\n");
  printf("Otherwise you can control ist functionality with these options:\n");
  printf("   long argument, short argument, value\n");
  printf("   --help,              ?              n/a\n");
  printf("   --clockwise         -c              default\n");
  printf("   --counter clockwise -b              counter clock wise\n");
  printf("   --numberOfSteps     -n              1..10000\n");
  printf("   --velocity          -v              slow,medium,fast\n");
  printf("   --fullStep          -s              default\n");
  printf("   --halfStep          -h              \n");
  }

int main(int argc, char ** argv) {
  enum stepTypeDef {full,half};
  enum stepTypeDef stepType;
  enum dirTypeDef  {cw,ccw};
  enum dirTypeDef  direction;
  enum velocityDef {slow,medium,fast};
  char *velocityString[]={"slow","medium","fast"};
  bool found;
  
  int i,noOfSteps;
  int velocity;
  
  int option,optIndex=0;

  
  if (argc > 1) {
    if (!strcmp(argv[1],"?")) {
      printUsage();
      return 1;
    }
  }
  /*
    setup default values
  */
  stepType  = full;
  direction = cw;
  velocity  = 1;
  noOfSteps = 512;
  
  while (true) {
    option = getopt_long (argc, argv, "?cbn:v:sh",
			  long_options, &optIndex);
    if (option == -1)
      break;
    
    switch (option) {	  

    case 's':
      /* this is for full step */
#ifdef DEBUG
      printf("stepper: step type set to full step\n");
#endif
      stepType = full;
      break;

    case 'h':
      /* this is for half step */
#ifdef DEBUG
      printf("stepper: step type set to half step\n");
#endif
      stepType = half;
      break;

    case 'c':
      /* this is for clockwise counter clockwise*/
#ifdef DEBUG
      printf("stepper: clockwise movement\n");
#endif
      stepType = half;
      break;
      
    case 'b':
      /* this is for counter clockwise movement */
#ifdef DEBUG
      printf("stepper: counter clockwise movement\n");
#endif
      stepType = half;
      
    case 'n':
      /* no of steps */
#ifdef DEBUG
      printf("stepper: no of steps: %s\n",optarg);
#endif
      stepType = half;
      break;

    case 'v':
      /* this is for counter clockwise movement */
#ifdef DEBUG
      printf("stepper: velocity: %s\n",optarg);
#endif
      found = false;
      for(i=slow;i<=fast;i++)
	if (!strcmp(optarg,velocityString[i])) {
	  found = true;
	  velocity = i;
	}
      if (!found) {
	printf("stepper: velocity string %s is unknown\n",optarg);
	printf("it must be one of \"slow\" \"medium\" or \"fast\"\n");
	printf("keeping default value which is \"fast\"\n");
      }
      printf("stepper: velocity is %s\n",velocityString[velocity]);
      break;

    default:
      printUsage();
    }
  }
}
